@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2000, 2005, 2007, 2010-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -69,6 +68,8 @@ extern int mu_observable_attach    (mu_observable_t, size_t type,
 				    mu_observer_t observer);
 extern int mu_observable_detach    (mu_observable_t, mu_observer_t observer);
 extern int mu_observable_notify    (mu_observable_t, int type, void *data);
+int mu_observable_wants (mu_observable_t observable, int type);
+
 
 #ifdef __cplusplus
 }
