@@ -1,6 +1,5 @@
-/* Returns true if SCHEME represents a local (autodetect) mail folder.  */
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -19,7 +18,8 @@
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
-
+#include <stdlib.h>
+#include <string.h>
 #include <mailutils/url.h>
 
 /* Returns true if SCHEME represents a local (autodetect) mail folder.  */
@@ -33,4 +33,29 @@ mu_scheme_autodetect_p (mu_url_t url)
     }
   return 0;
 }
-    
+
+static int accuracy = MU_AUTODETECT_ACCURACY_AUTO;
+
+void
+mu_set_autodetect_accuracy (int v)
+{
+  accuracy = v;
+}
+
+int
+mu_autodetect_accuracy (void)
+{
+  if (accuracy == MU_AUTODETECT_ACCURACY_AUTO)
+    {
+      char *p = getenv ("MU_AUTODETECT_ACCURACY");
+      if (!p || strcmp (p, "default") == 0)
+	accuracy = MU_AUTODETECT_ACCURACY_DEFAULT;
+      else if (strcmp (p, "fast") == 0)
+	accuracy = MU_AUTODETECT_ACCURACY_FAST;
+      else if (strcmp (p, "max") == 0)
+	accuracy = MU_AUTODETECT_ACCURACY_MAX;
+      else
+	accuracy = atoi (p);
+    }
+  return accuracy;
+}
