@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2020 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@ mu_utc_offset (void)
   struct tm ltm = *localtime (&t);
   struct tm gtm = *gmtime (&t);
   int d = TMSEC (&ltm) - TMSEC (&gtm);
-  if (!(ltm.tm_year = gtm.tm_year
+  if (!(ltm.tm_year == gtm.tm_year
 	&& ltm.tm_mon == gtm.tm_mon
 	&& ltm.tm_mday == gtm.tm_mday))
     d += 86400;
