@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -44,28 +44,13 @@
 #include <mailutils/sys/message.h>
 #include <mailutils/sys/stream.h>
 
-#ifndef TRUE
-#define TRUE (1)
-#define FALSE (0)
-#endif
-
-#define CT_MULTIPART_DIGEST "multipart/digest"
-#define CT_MULTIPART_DIGEST_LEN (sizeof (CT_MULTIPART_DIGEST) - 1)
+static char default_content_type[] = "text/plain; charset=us-ascii";
 
 /* TODO:
  *  Need to prevent re-entry into mime lib, but allow non-blocking re-entry
  *  into lib.
  */
 
-static int
-_mime_is_multipart_digest (mu_mime_t mime)
-{
-  if (mime->content_type)
-    return mu_c_strncasecmp (CT_MULTIPART_DIGEST, mime->content_type,
-			     CT_MULTIPART_DIGEST_LEN) == 0;
-  return 0;
-}
-
 static int
 _mime_append_part (mu_mime_t mime, mu_message_t msg,
 		   size_t offset, size_t len, size_t lines)
@@ -115,7 +100,7 @@ _mime_append_part (mu_mime_t mime, mu_message_t msg,
 	   mu_header_get_value (hdr, MU_HEADER_CONTENT_TYPE, NULL,
 				0, &size)) != 0 || size == 0)
 	{
-	  if (_mime_is_multipart_digest (mime))
+	  if (mu_c_strcasecmp (mime->content_type->subtype, "digest") == 0)
 	    mu_header_set_value (hdr,
 				 MU_HEADER_CONTENT_TYPE, "message/rfc822", 0);
 	  else
@@ -139,147 +124,171 @@ _mime_append_part (mu_mime_t mime, mu_message_t msg,
   return 0;
 }
 
-#define _ISSPECIAL(c) (						 \
-    ((c) == '(') || ((c) == ')') || ((c) == '<') || ((c) == '>') \
-    || ((c) == '@') || ((c) == ',') || ((c) == ';') || ((c) == ':') \
-    || ((c) == '\\') || ((c) == '.') || ((c) == '[') \
-    || ((c) == ']') )
-
-static void
-_mime_munge_content_header (char *field_body)
-{
-  char *p, *e, *str = field_body;
-  int quoted = 0;
-
-  mu_str_stripws (field_body);
-
-  if ((e = strchr (str, ';')) == NULL)
-    return;
-  while (*e == ';')
-    {
-      p = e;
-      e++;
-      while (*e && mu_isspace (*e)) /* remove space up to param */
-	e++;
-      memmove (p + 1, e, strlen (e) + 1);
-      e = p + 1;
-
-      while (*e && *e != '=')	/* find end of value */
-	e++;
-      e = p = e + 1;
-      while (*e
-	     && (quoted
-		 || (!_ISSPECIAL (*e) && !mu_isspace (*e))))
-	{
-	  if (*e == '\\')
-	    {			/* escaped */
-	      memmove (e, e + 1, strlen (e));
-	    }
-	  else if (*e == '\"')
-	    quoted = ~quoted;
-	  e++;
-	}
-    }
-}
-
-static char    *
-_mime_get_param (char *field_body, const char *param, int *len)
-{
-  char *str, *p, *v, *e;
-  int quoted = 0, was_quoted;
-
-  if (len == NULL || (str = field_body) == NULL)
-    return NULL;
-
-  p = strchr (str, ';');
-  while (p)
-    {
-      p++;
-      if ((v = strchr (p, '=')) == NULL)
-	break;
-      *len = 0;
-      v = e = v + 1;
-      was_quoted = 0;
-      while (*e
-	     && (quoted
-		 || (!_ISSPECIAL (*e) && !mu_isspace (*e))))
-	{			/* skip pass value and calc len */
-	  if (*e == '\"')
-	    quoted = ~quoted, was_quoted = 1;
-	  else
-	    (*len)++;
-	  e++;
-	}
-      if (mu_c_strncasecmp (p, param, strlen (param)))
-	{			/* no match jump to next */
-	  p = strchr (e, ';');
-	  continue;
-	}
-      else
-	return was_quoted ? v + 1 : v;	/* return unquoted value */
-    }
-  return NULL;
-}
-
-static int
-_mime_setup_buffers (mu_mime_t mime)
-{
-  if (mime->cur_buf == NULL
-      && (mime->cur_buf = malloc (mime->buf_size)) == NULL)
-    {
-      return ENOMEM;
-    }
-  if (mime->cur_line == NULL
-      && (mime->cur_line = calloc (mime->line_size, 1)) == NULL)
-    {
-      free (mime->cur_buf);
-      return ENOMEM;
-    }
-  return 0;
-}
-
 static void
 _mime_append_header_line (mu_mime_t mime)
 {
-  if (mime->header_length + mime->line_ndx > mime->header_buf_size)
+  if (mime->header_length + mime->line_length > mime->header_buf_size)
     {
       char *nhb;
 
       if ((nhb = realloc (mime->header_buf,
-			  mime->header_length + mime->line_ndx + 128)) == NULL)
+			  mime->header_length + mime->line_length + 128)) == NULL)
 	return;
       mime->header_buf = nhb;
-      mime->header_buf_size = mime->header_length + mime->line_ndx + 128;
+      mime->header_buf_size = mime->header_length + mime->line_length + 128;
     }
   memcpy (mime->header_buf + mime->header_length, mime->cur_line,
-	  mime->line_ndx);
-  mime->header_length += mime->line_ndx;
+	  mime->line_length);
+  mime->header_length += mime->line_length;
 }
 
+int
+mu_mime_sget_content_type (mu_mime_t mime, const char **value)
+{
+  if (!mime)
+    return EINVAL;
+  if (!mime->content_type)
+    return MU_ERR_NOENT;
+  if (value)
+    *value = mime->content_type->type;
+  return 0;
+}
+
+int
+mu_mime_aget_content_type (mu_mime_t mime, char **value)
+{
+  char const *s;
+  int rc = mu_mime_sget_content_type (mime, &s);
+  if (rc == 0 && value)
+    {
+      if ((*value = strdup (s)) == NULL)
+	return errno;
+    }
+  return 0;
+}
+
+int
+mu_mime_sget_content_subtype (mu_mime_t mime, const char **value)
+{
+  if (!mime)
+    return EINVAL;
+  if (!mime->content_type)
+    return MU_ERR_NOENT;
+  if (value)
+    *value = mime->content_type->subtype;
+  return 0;
+}
+
+int
+mu_mime_aget_content_subtype (mu_mime_t mime, char **value)
+{
+  char const *s;
+  int rc = mu_mime_sget_content_subtype (mime, &s);
+  if (rc == 0 && value)
+    {
+      if ((*value = strdup (s)) == NULL)
+	return errno;
+    }
+  return 0;
+}
+
+int
+mu_mime_content_type_get_param (mu_mime_t mime, char const *name,
+				const char **value)
+{
+  struct mu_mime_param *p;
+  int rc;
+
+  if (!mime || !name)
+    return EINVAL;
+
+  if (!mime->content_type)
+    return MU_ERR_NOENT;
+  
+  rc = mu_assoc_lookup (mime->content_type->param, name, &p);
+  if (rc == 0 && value)
+    *value = p->value;
+  return rc;
+}
+
+int
+mu_mime_content_type_set_param (mu_mime_t mime, char const *name,
+				const char *value)
+{
+  int rc;
+  struct mu_mime_param **pparam;
+  char *vcopy;
+  
+  if (!mime || !mime->content_type || !name)
+    return EINVAL;
+
+  if (!value)
+    return mu_assoc_remove (mime->content_type->param, name);
+  
+  if ((vcopy = strdup (value)) == NULL)
+    return ENOMEM;
+  
+  rc = mu_assoc_install_ref (mime->content_type->param, name, &pparam);
+  switch (rc)
+    {
+    case 0:
+      if ((*pparam = malloc (sizeof **pparam)) == NULL)
+	{
+	  free (vcopy);
+	  return ENOMEM;
+	}
+      break;
+
+    case MU_ERR_EXISTS:
+      free ((*pparam)->value);
+      break;
+
+    default:
+      return rc;
+    }
+  (*pparam)->value = vcopy;
+  return 0;
+}
+
+enum
+  {
+    BOUNDARY_NO_MATCH,
+    BOUNDARY_MATCH,
+    BOUNDARY_CLOSE
+  };
+
+static int
+match_boundary (char const *line, size_t llen,
+		char const *boundary, size_t blen)
+{
+  if (line[llen-1] == '\n')
+    llen--;
+  if (llen >= blen + 2 &&
+      memcmp (line, "--", 2) == 0 &&
+      memcmp (line + 2, boundary, blen) == 0)
+    {
+      if (llen == blen + 2)
+	return BOUNDARY_MATCH;
+      if (llen == blen + 4 && memcmp (line + blen + 2, "--", 2) == 0)
+	return BOUNDARY_CLOSE;
+    }
+  return BOUNDARY_NO_MATCH;
+}  
+
 static int
 _mime_parse_mpart_message (mu_mime_t mime)
 {
-  char *cp;
   size_t blength, mb_length, mb_offset, mb_lines;
   int ret;
-  size_t nbytes;
-
+  int match;
+  
   if (!(mime->flags & MIME_PARSER_ACTIVE))
     {
-      char *boundary;
-      int len;
-
-      if ((ret = _mime_setup_buffers (mime)) != 0)
+      ret = mu_mime_content_type_get_param (mime, "boundary", &mime->boundary);
+      if (ret)
 	return ret;
-      if ((boundary = _mime_get_param (mime->content_type, "boundary", &len))
-	  == NULL)
-	return EINVAL;
-      if ((mime->boundary = calloc (1, len + 1)) == NULL)
-	return ENOMEM;
-      strncpy (mime->boundary, boundary, len);
-
       mime->cur_offset = 0;
-      mime->line_ndx = 0;
+      mime->line_length = 0;
       mime->parser_state = MIME_STATE_SCAN_BOUNDARY;
       mime->flags |= MIME_PARSER_ACTIVE;
     }
@@ -289,126 +298,67 @@ _mime_parse_mpart_message (mu_mime_t mime)
   blength = strlen (mime->boundary);
 
   mu_stream_seek (mime->stream, mime->cur_offset, MU_SEEK_SET, NULL);
-  while ((ret = mu_stream_read (mime->stream, mime->cur_buf, mime->buf_size,
-				&nbytes)) == 0 && nbytes)
+  while (mime->parser_state != MIME_STATE_END &&
+	 (ret = mu_stream_getline (mime->stream,
+				   &mime->cur_line, &mime->line_size,
+				   &mime->line_length)) == 0 && mime->line_length)
     {
-      cp = mime->cur_buf;
-
-      while (1)
+      switch (mime->parser_state)
 	{
-	  if (nbytes)
-	    mime->cur_line[mime->line_ndx] = *cp;
-
-	  if (mime->parser_state == MIME_STATE_SCAN_BOUNDARY
-	      && (nbytes == 0 || *cp == '\n'))
+	case MIME_STATE_SCAN_BOUNDARY:
+	  if ((match = match_boundary (mime->cur_line, mime->line_length,
+				       mime->boundary, blength))
+	      != BOUNDARY_NO_MATCH)
 	    {
-	      char *cp2 = mime->cur_line[0] == '\n'
-		            ? mime->cur_line + 1 : mime->cur_line;
-	      if (mime->line_ndx >= blength
-		  && ((memcmp (cp2, "--", 2) == 0
-		       && memcmp (cp2 + 2, mime->boundary, blength) == 0)
-		      || memcmp (cp2, mime->boundary, blength) == 0))
+	      mime->parser_state = MIME_STATE_HEADERS;
+	      if (mime->cur_offset == mb_offset)
+		mb_length = 0;
+	      else
+		mb_length = mime->cur_offset - mb_offset - 1;
+	      if (mime->header_length)
+		/* this skips the preamble */
 		{
-		  mime->parser_state = MIME_STATE_HEADERS;
-		  mime->flags &= ~MIME_PARSER_HAVE_CR;
-		  mb_length = mime->cur_offset 
-			                   - mb_offset
-			                   - mime->line_ndx;
-		  if (mime->header_length)
-		    /* this skips the preamble */
-		    {
-		      /* RFC 1521 [Page 30]:
-			 NOTE: The CRLF preceding the encapsulation
-			 line is conceptually attached to the boundary
-			 so that it is possible to have a part that
-			 does not end with a CRLF (line break). Body
-			 parts that must be considered to end with line
-			 breaks, therefore, must have two CRLFs
-			 preceding the encapsulation line, the first
-			 of which is part of the preceding body part,
-			 and the second of which is part of the
-			 encapsulation boundary. */
-		      
-		      if (mb_lines)
-			/* to prevent negative values in case of a
-			   malformed message */
-			mb_lines--;
-		      
-		      _mime_append_part (mime, NULL,
-					 mb_offset, mb_length,
-					 mb_lines);
-		    }
+		  /* RFC 1521 [Page 30]:
+		     NOTE: The CRLF preceding the encapsulation
+		     line is conceptually attached to the boundary
+		     so that it is possible to have a part that
+		     does not end with a CRLF (line break). Body
+		     parts that must be considered to end with line
+		     breaks, therefore, must have two CRLFs
+		     preceding the encapsulation line, the first
+		     of which is part of the preceding body part,
+		     and the second of which is part of the
+		     encapsulation boundary. */
 		  
-		  if ((&mime->cur_line[mime->line_ndx] - cp2 - 1 > blength
-		       && memcmp (cp2 + blength + 2, "--", 2) == 0)
-		      || (&mime->cur_line[mime->line_ndx] - cp2 - 1 == blength
-			  && memcmp (cp2 + blength, "--", 2) == 0))
-		    {	/* last boundary */
-		      mime->parser_state = MIME_STATE_BEGIN_LINE;
-		      mime->header_length = 0;
-		    }
-		  else
-		    mime->line_ndx = -1; /* headers parsing requires
-					    empty line */
+		  if (mb_lines)
+		    /* to prevent negative values in case of a
+		       malformed message */
+		    mb_lines--;
+		  
+		  _mime_append_part (mime, NULL,
+				     mb_offset, mb_length, mb_lines);
 		}
-	      else if (nbytes)
-		{
-		  if (mime->header_length)
-		    mb_lines++;
-		  
-		  mime->line_ndx = 0;
-		  mime->cur_line[0] = *cp;	/* stay in this state but
-						   leave '\n' at begining */
+
+	      if (match == BOUNDARY_CLOSE)
+		{	/* last boundary */
+		  mime->parser_state = MIME_STATE_END;
+		  mime->header_length = 0;
 		}
 	    }
-
-	  if (nbytes == 0) 
-	    break;
+	  else
+	    mb_lines++;
+	  break;
 	  
-	  if (*cp == '\n')
+	case MIME_STATE_HEADERS:
+	  _mime_append_header_line (mime);
+	  if (mime->line_length == 1)
 	    {
-	      switch (mime->parser_state)
-		{
-		case MIME_STATE_BEGIN_LINE:
-		  mime->cur_line[0] = *cp;
-		  mime->line_ndx = 0;
-		  mime->parser_state = MIME_STATE_SCAN_BOUNDARY;
-		  break;
-
-		case MIME_STATE_SCAN_BOUNDARY:
-		  break;
-
-		case MIME_STATE_HEADERS:
-		  mime->line_ndx++;
-		  _mime_append_header_line (mime);
-		  if (mime->line_ndx == 1 || mime->cur_line[0] == '\r')
-		    {
-		      mime->parser_state = MIME_STATE_SCAN_BOUNDARY;
-		      mb_offset = mime->cur_offset + 1;
-		      mb_lines = 0;
-		    }
-		  mime->line_ndx = -1;
-		  break;
-		}
+	      mime->parser_state = MIME_STATE_SCAN_BOUNDARY;
+	      mb_offset = mime->cur_offset + 1;
+	      mb_lines = 0;
 	    }
-
-	  mime->line_ndx++;
-	  if (mime->line_ndx >= mime->line_size)
-	    {
-	      size_t newsize = mime->line_size + MIME_MAX_HDR_LEN;
-	      char *p = realloc (mime->cur_line, newsize);
-	      if (!p)
-		{
-		  ret = ENOMEM;
-		  break;
-		}
-	      mime->cur_line = p;
-	      mime->line_size = newsize;
-	    }
-	  mime->cur_offset++;
-	  nbytes--;
-	  cp++;
 	}
+      mime->cur_offset += mime->line_length;
     }
   mime->body_lines = mb_lines;
   mime->body_length = mb_length;
@@ -453,19 +403,12 @@ _mimepart_body_lines (mu_body_t body, size_t *plines)
 }
 
 /*------ Mime message/header functions for CREATING multipart message -----*/
-static int
-retain_charset (char const *name, void *value MU_ARG_UNUSED,
-		void *data MU_ARG_UNUSED)
-{
-  return strcmp (name, "charset") != 0;
-}
 		 
 static int
 _mime_set_content_type (mu_mime_t mime)
 {
   const char  *content_type;
   mu_header_t hdr = NULL;
-  size_t size;
   int ret;
 
   /* Delayed the creation of the header 'til they create the final message via
@@ -474,13 +417,27 @@ _mime_set_content_type (mu_mime_t mime)
     return 0;
   if (mime->nmtp_parts > 1)
     {
+      char const *subtype;
       char *cstr;
-      
+
       if (mime->flags & MIME_ADDED_MULTIPART_CT)
 	return 0;
-      if (mime->flags & MU_MIME_MULTIPART_MIXED)
-	content_type = "multipart/mixed; boundary=";
-      else
+
+      ret = mu_mime_sget_content_subtype (mime, &subtype);
+      switch (ret)
+	{
+	case 0:
+	  break;
+	  
+	case MU_ERR_NOENT:
+	  subtype = MU_MIME_CONTENT_SUBTYPE_MIXED;
+	  break;
+
+	default:
+	  return ret;
+	}
+
+      if (mu_c_strcasecmp (subtype, MU_MIME_CONTENT_SUBTYPE_ALTERNATIVE) == 0)
 	{
 	  size_t i;
 
@@ -505,9 +462,6 @@ _mime_set_content_type (mu_mime_t mime)
 		    {
 		      char *type;
 		      
-		      mu_assoc_mark (ct->param, retain_charset, NULL);
-		      mu_assoc_sweep (ct->param);
-
 		      rc = mu_asprintf (&type, "%s/%s", ct->type, ct->subtype);
 		      if (rc == 0)
 			{
@@ -518,32 +472,20 @@ _mime_set_content_type (mu_mime_t mime)
 			}
 		      mu_content_type_destroy (&ct);
 		    }
+		  free (val);
 		}
 	    }
-	  
-	  content_type = "multipart/alternative; boundary=";
 	}
-      if (mime->boundary == NULL)
-	{
-	  char boundary[128];
-	  
-	  snprintf (boundary, sizeof boundary, "%ld-%ld=:%ld",
-		    (long) random (), (long) time (0), (long) getpid ());
-	  if ((mime->boundary = strdup (boundary)) == NULL)
-	    return ENOMEM;
-	}
-      size = strlen (content_type) + 2 + strlen (mime->boundary) + 1;
-      cstr = malloc (size);
-      if (!cstr)
-	return ENOMEM;
-      strcpy (cstr, content_type);
-      strcat (cstr, "\"");
-      strcat (cstr, mime->boundary);
-      strcat (cstr, "\"");
-      mime->flags |= MIME_ADDED_MULTIPART_CT;
 
-      ret = mu_header_set_value (mime->hdrs, MU_HEADER_CONTENT_TYPE, cstr, 1);
-      free (cstr);
+      ret = mu_content_type_format (mime->content_type, &cstr);
+      if (ret == 0)
+	{
+	  ret = mu_header_set_value (mime->hdrs, MU_HEADER_CONTENT_TYPE,
+				     cstr, 1);	  
+	  free (cstr);
+	  if (ret == 0)
+	    mime->flags |= MIME_ADDED_MULTIPART_CT;
+	}
     }
   else
     {
@@ -557,7 +499,7 @@ _mime_set_content_type (mu_mime_t mime)
       if (hdr == NULL
 	  || mu_header_sget_value (hdr, MU_HEADER_CONTENT_TYPE,
 				   &content_type))
-	content_type = "text/plain; charset=us-ascii";
+	content_type = default_content_type;
 
       ret = mu_header_set_value (mime->hdrs, MU_HEADER_CONTENT_TYPE,
 				 content_type, 1);
@@ -571,8 +513,8 @@ _mime_set_content_type (mu_mime_t mime)
 	  /* if the only part contains a transfer-encoding
 	     field, set it on the message header too */
 	  if (mu_header_sget_value (hdr,
-				MU_HEADER_CONTENT_TRANSFER_ENCODING,
-				&content_te) == 0)
+				    MU_HEADER_CONTENT_TRANSFER_ENCODING,
+				    &content_te) == 0)
 	    ret = mu_header_set_value (mime->hdrs,
 				       MU_HEADER_CONTENT_TRANSFER_ENCODING,
 				       content_te, 1);
@@ -608,7 +550,9 @@ _mime_part_size (mu_mime_t mime, size_t *psize)
     return ret;
   if (mime->nmtp_parts == 1)
     {
-      ret = mu_message_size (mime->mtp_parts[0]->msg, &total);
+      mu_body_t part_body;
+      if ((ret = mu_message_get_body (mime->mtp_parts[0]->msg, &part_body)) == 0)
+	ret = mu_body_size (part_body, &total);
     }
   else
     {
@@ -626,7 +570,8 @@ _mime_part_size (mu_mime_t mime, size_t *psize)
 	}
       total += 2; /* ending boundary line */
     }
-  *psize = total;
+  if (ret == 0)
+    *psize = total;
   return ret;
 }
 
@@ -727,69 +672,12 @@ _mime_body_stream_read (mu_stream_t stream, char *buf, size_t buflen,
   
   if ((ret = _mime_set_content_type (mime)) == 0)
     {
-      do
+      if (mime->nmtp_parts == 1)
 	{
-	  size_t part_nbytes = 0;
-
-	  if (buflen == 0)
-	    break;
-	  if (mime->nmtp_parts > 1)
-	    {
-	      size_t len;
-	      
-	      if (mime->flags & MIME_INSERT_BOUNDARY)
-		{
-		  if ((mime->flags & MIME_ADDING_BOUNDARY) == 0)
-		    {
-		      mime->boundary_len = strlen (mime->boundary);
-		      mime->preamble = 2;
-		      if (mime->cur_part == mime->nmtp_parts)
-			mime->postamble = 2;
-		      mime->flags |= MIME_ADDING_BOUNDARY;
-		    }
-		  while (mime->preamble)
-		    {
-		      mime->preamble--;
-		      ADD_CHAR (buf, '-', mime->cur_offset, buflen,
-				total, nbytes);
-		    }
-		  len = strlen (mime->boundary) - mime->boundary_len;
-		  while (mime->boundary_len)
-		    {
-		      mime->boundary_len--;
-		      ADD_CHAR (buf,
-				mime->boundary[len++],
-				mime->cur_offset, buflen,
-				total, nbytes);
-		    }
-		  while (mime->postamble)
-		    {
-		      mime->postamble--;
-		      ADD_CHAR (buf, '-', mime->cur_offset, buflen,
-				total, nbytes);
-		    }
-		  mime->flags &=
-		    ~(MIME_INSERT_BOUNDARY | MIME_ADDING_BOUNDARY);
-		  mime->part_offset = 0;
-		  ADD_CHAR (buf, '\n', mime->cur_offset, buflen,
-			    total, nbytes);
-		}
-
-	      if (!mime->part_stream)
-		{
-		  if (mime->cur_part >= mime->nmtp_parts)
-		    {
-		      *nbytes = total;
-		      return 0;
-		    }
-		  ret = mu_message_get_streamref (mime->mtp_parts[mime->cur_part]->msg,
-						  &mime->part_stream);
-		}
-	    }
-	  else if (!mime->part_stream)
+	  if (!mime->part_stream)
 	    {
 	      mu_body_t part_body;
-
+	      
 	      if (mime->cur_part >= mime->nmtp_parts)
 		{
 		  *nbytes = total;
@@ -799,39 +687,123 @@ _mime_body_stream_read (mu_stream_t stream, char *buf, size_t buflen,
 				   &part_body);
 	      ret = mu_body_get_streamref (part_body, &mime->part_stream);
 	    }
-	  if (ret)
-	    break;
-	  ret = mu_stream_seek (mime->part_stream, mime->part_offset,
-				MU_SEEK_SET, NULL);
-	  if (ret)
+	  if (ret == 0)
 	    {
-	      mu_stream_destroy (&mime->part_stream);
-	      break;
-	    }
-	  while (buflen > 0 &&
-		 (ret = mu_stream_read (mime->part_stream, buf, buflen,
-					&part_nbytes)) == 0)
-	    {
-	      if (part_nbytes)
+	      size_t part_nbytes = 0;
+	      while (buflen > 0 &&
+		     (ret = mu_stream_read (mime->part_stream, buf, buflen,
+					    &part_nbytes)) == 0)
 		{
-		  mime->part_offset += part_nbytes;
-		  mime->cur_offset += part_nbytes;
-		  total += part_nbytes;
-		  buflen -= part_nbytes;
-		  buf += part_nbytes;
+		  if (part_nbytes)
+		    {
+		      mime->part_offset += part_nbytes;
+		      mime->cur_offset += part_nbytes;
+		      total += part_nbytes;
+		      buflen -= part_nbytes;
+		      buf += part_nbytes;
+		    }
+		  else 
+		    {
+		      mu_stream_destroy (&mime->part_stream);
+		      mime->cur_part++;
+		      break;
+		    }
 		}
-	      else 
-		{
-		  mu_stream_destroy (&mime->part_stream);
-		  mime->flags |= MIME_INSERT_BOUNDARY;
-		  mime->cur_part++;
-		  ADD_CHAR (buf, '\n', mime->cur_offset, buflen,
-			    total, nbytes);
-		  break;
-		}
-	    }
+	    }	
 	}
-      while (ret == 0 && mime->cur_part <= mime->nmtp_parts);
+      else
+	do
+	  {
+	    size_t part_nbytes = 0;
+
+	    if (buflen == 0)
+	      break;
+	    else 
+	      {
+		size_t len;
+	      
+		if (mime->flags & MIME_INSERT_BOUNDARY)
+		  {
+		    if ((mime->flags & MIME_ADDING_BOUNDARY) == 0)
+		      {
+			mime->boundary_len = strlen (mime->boundary);
+			mime->preamble = 2;
+			if (mime->cur_part == mime->nmtp_parts)
+			  mime->postamble = 2;
+			mime->flags |= MIME_ADDING_BOUNDARY;
+		      }
+		    while (mime->preamble)
+		      {
+			mime->preamble--;
+			ADD_CHAR (buf, '-', mime->cur_offset, buflen,
+				  total, nbytes);
+		      }
+		    len = strlen (mime->boundary) - mime->boundary_len;
+		    while (mime->boundary_len)
+		      {
+			mime->boundary_len--;
+			ADD_CHAR (buf,
+				  mime->boundary[len++],
+				  mime->cur_offset, buflen,
+				  total, nbytes);
+		      }
+		    while (mime->postamble)
+		      {
+			mime->postamble--;
+			ADD_CHAR (buf, '-', mime->cur_offset, buflen,
+				  total, nbytes);
+		      }
+		    mime->flags &=
+		      ~(MIME_INSERT_BOUNDARY | MIME_ADDING_BOUNDARY);
+		    mime->part_offset = 0;
+		    ADD_CHAR (buf, '\n', mime->cur_offset, buflen,
+			      total, nbytes);
+		  }
+		
+		if (!mime->part_stream)
+		  {
+		    if (mime->cur_part >= mime->nmtp_parts)
+		      {
+			*nbytes = total;
+			return 0;
+		      }
+		    ret = mu_message_get_streamref (mime->mtp_parts[mime->cur_part]->msg,
+						    &mime->part_stream);
+		  }
+	      }
+	    
+	    if (ret)
+	      break;
+	    ret = mu_stream_seek (mime->part_stream, mime->part_offset,
+				  MU_SEEK_SET, NULL);
+	    if (ret)
+	      {
+		mu_stream_destroy (&mime->part_stream);
+		break;
+	      }
+	    while (buflen > 0 &&
+		   (ret = mu_stream_read (mime->part_stream, buf, buflen,
+					  &part_nbytes)) == 0)
+	      {
+		if (part_nbytes)
+		  {
+		    mime->part_offset += part_nbytes;
+		    mime->cur_offset += part_nbytes;
+		    total += part_nbytes;
+		    buflen -= part_nbytes;
+		    buf += part_nbytes;
+		  }
+		else 
+		  {
+		    mu_stream_destroy (&mime->part_stream);
+		    mime->flags |= MIME_INSERT_BOUNDARY;
+		    mime->cur_part++;
+		    ADD_CHAR (buf, '\n', mime->cur_offset, buflen, total, nbytes);
+		    break;
+		  }
+	      }
+	  }
+	while (ret == 0 && mime->cur_part <= mime->nmtp_parts);
     }
   if (ret)
     mu_stream_destroy (&mime->part_stream);
@@ -942,80 +914,190 @@ _mime_body_lines (mu_body_t body, size_t *plines)
 int
 mu_mime_create (mu_mime_t *pmime, mu_message_t msg, int flags)
 {
-  mu_mime_t mime = NULL;
   int ret = 0;
-  size_t size;
-  mu_body_t body;
   
   if (pmime == NULL)
     return EINVAL;
   
-  switch (flags & MIME_MULTIPART_FLAGS)
-    {
-    case MIME_MULTIPART_FLAGS:
-      return EINVAL;
+  if ((flags & MIME_MULTIPART_FLAGS) == MIME_MULTIPART_FLAGS
+      || (flags & ~MIME_MULTIPART_FLAGS) != 0)
+    return EINVAL;
 
-    case 0:
-      flags |= MU_MIME_MULTIPART_MIXED;
-    }
-  
-  *pmime = NULL;
-  if ((mime = calloc (1, sizeof (*mime))) == NULL)
-    return ENOMEM;
   if (msg)
     {
-      if ((ret = mu_message_get_header (msg, &mime->hdrs)) == 0)
+      mu_mime_t mime = NULL;
+      mu_body_t body;
+      
+      *pmime = NULL;
+      if ((mime = calloc (1, sizeof (*mime))) == NULL)
+	return ENOMEM;
+      if (msg)
 	{
-	  if ((ret = mu_header_get_value (mime->hdrs,
-					  MU_HEADER_CONTENT_TYPE,
-					  NULL, 0, &size)) == 0 && size)
+	  if ((ret = mu_message_get_header (msg, &mime->hdrs)) == 0)
 	    {
-	      if ((mime->content_type = malloc (size + 1)) == NULL)
-		ret = ENOMEM;
-	      else if ((ret = mu_header_get_value (mime->hdrs,
-						   MU_HEADER_CONTENT_TYPE,
-						   mime->content_type,
-						   size + 1, 0)) == 0)
-		_mime_munge_content_header (mime->content_type);
-	    }
-	  else
-	    {
-	      if (ret == MU_ERR_NOENT)
+	      char *buf;
+	      if ((ret = mu_header_aget_value_unfold (mime->hdrs,
+						      MU_HEADER_CONTENT_TYPE,
+						      &buf)) == 0)
 		{
-		  ret = 0;
-		  if ((mime->content_type =
-		       strdup ("text/plain; charset=us-ascii")) == NULL)
-		    /* default as per spec. */
-		    ret = ENOMEM;
+		  ret = mu_content_type_parse (buf, NULL, &mime->content_type);
+		  free (buf);
+		}
+	      else if (ret == MU_ERR_NOENT)
+		{
+		  ret = mu_content_type_parse (default_content_type,
+					       NULL, &mime->content_type);
+		}
+	      if (ret == 0)
+		{
+		  mime->msg = msg;
+		  mu_message_get_body (msg, &body);
+		  mu_body_get_streamref (body, &mime->stream);
 		}
 	    }
-	  if (ret == 0)
-	    {
-	      mime->msg = msg;
-	      mime->buf_size = MIME_DFLT_BUF_SIZE;
-	      mime->line_size = MIME_MAX_HDR_LEN;
-	      mu_message_get_body (msg, &body);
-	      mu_body_get_streamref (body, &mime->stream);
-	    }
+	}
+      if (ret != 0)
+	{
+	  mu_content_type_destroy (&mime->content_type);
+	  free (mime);
+	}
+      else
+	{
+	  mime->flags = 0;
+	  mime->ref_count = 1;
+	  *pmime = mime;
 	}
     }
   else
     {
-      mime->flags |= MIME_NEW_MESSAGE;
+      ret = mu_mime_create_multipart (pmime,
+				      flags == 0
+				        || (flags & MU_MIME_MULTIPART_MIXED)
+				       ? MU_MIME_CONTENT_SUBTYPE_MIXED
+				       : MU_MIME_CONTENT_SUBTYPE_ALTERNATIVE,
+				      NULL);
     }
-  if (ret != 0)
+
+  return ret;
+}
+
+static int
+param_copy (char const *name, void *vptr, void *call_data)
+{
+  mu_assoc_t a = call_data;
+  struct mu_mime_param *val = vptr;
+  struct mu_mime_param *p, **pptr;
+  int rc;
+  
+  rc = mu_assoc_install_ref2 (a, name, &pptr, NULL);
+  if (rc == MU_ERR_EXISTS)
+    return 0; /* Ignore duplicates */
+  if (rc)
+    return rc;
+  
+  if ((p = malloc (sizeof *p)) == NULL)
+    return ENOMEM;
+  if (val->lang)
     {
-      if (mime->content_type)
-	free (mime->content_type);
-      free (mime);
+      if ((p->lang = strdup (val->lang)) == NULL)
+	{
+	  mu_mime_param_free (p);
+	  return ENOMEM;
+	}
     }
   else
+    p->lang = NULL;
+
+  if (val->cset)
     {
-      mime->flags |= (flags & MIME_FLAG_MASK);
-      mime->ref_count = 1;
-      *pmime = mime;
+      if ((p->cset = strdup (val->cset)) == NULL)
+	{
+	  mu_mime_param_free (p);
+	  return ENOMEM;
+	}
     }
-  return ret;
+  else
+    p->cset = NULL;
+  
+  if ((p->value = strdup (val->value)) == NULL)
+    {
+      mu_mime_param_free (p);
+      return ENOMEM;
+    }
+  *pptr = p;
+  return 0;
+}
+
+int
+mu_mime_create_multipart (mu_mime_t *pmime, char const *subtype,
+			  mu_assoc_t param)
+{
+  int rc;
+  mu_mime_t mime;
+  char boundary[128];
+  struct mu_mime_param *p;
+  
+  if (pmime == NULL)
+    return MU_ERR_OUT_PTR_NULL;
+  if ((mime = calloc (1, sizeof (*mime))) == NULL)
+    return ENOMEM;
+  mime->flags |= MIME_NEW_MESSAGE;
+  mime->ref_count = 1;
+	  
+  rc = mu_content_type_parse ("multipart/mixed", NULL, &mime->content_type);
+  if (rc)
+    {
+      free (mime);
+      return rc;
+    }
+  if (subtype)
+    {
+      free (mime->content_type->subtype);
+      if ((mime->content_type->subtype = strdup (subtype)) == NULL)
+	{
+	  mu_mime_destroy (&mime);
+	  return errno;
+	}
+    }
+
+  snprintf (boundary, sizeof boundary, "%ld-%ld=:%ld",
+	    (long) random (), (long) time (0), (long) getpid ());
+  
+  p = calloc (1, sizeof (*p));
+  if (!p)
+    {
+      mu_mime_destroy (&mime);
+      return rc;
+    }
+
+  if ((p->value = strdup (boundary)) == NULL)
+    {
+      free (p);
+      mu_mime_destroy (&mime);
+      return errno;
+    }
+  rc = mu_assoc_install (mime->content_type->param, "boundary", p);
+  if (rc)
+    {
+      free (p->value);
+      free (p);
+      mu_mime_destroy (&mime);
+      return rc;
+    }
+  mime->boundary = p->value;
+
+  if (param)
+    {
+      rc = mu_assoc_foreach (param, param_copy, mime->content_type->param);
+      if (rc)
+	{
+	  mu_mime_destroy (&mime);
+	  return rc;
+	}
+    }
+  
+  *pmime = mime;
+  return 0;
 }
 
 void
@@ -1044,16 +1126,9 @@ _mu_mime_free (mu_mime_t mime)
   mu_stream_destroy (&mime->part_stream);
   if (mime->msg && mime->flags & MIME_NEW_MESSAGE)
     mu_message_destroy (&mime->msg, mime);
-  if (mime->content_type)
-    free (mime->content_type);
-  if (mime->cur_buf)
-    free (mime->cur_buf);
-  if (mime->cur_line)
-    free (mime->cur_line);
-  if (mime->boundary)
-    free (mime->boundary);
-  if (mime->header_buf)
-    free (mime->header_buf);
+  mu_content_type_destroy (&mime->content_type);
+  free (mime->cur_line);
+  free (mime->header_buf);
   free (mime);
 }
 
@@ -1226,8 +1301,6 @@ mu_mime_to_message (mu_mime_t mime, mu_message_t *pmsg)
 int
 mu_mime_is_multipart (mu_mime_t mime)
 {
-  if (mime->content_type)
-    return (mu_c_strncasecmp ("multipart", mime->content_type,
-			      strlen ("multipart")) ? 0 : 1);
-  return 0;
+  return (mime->content_type
+	  && mu_c_strcasecmp (mime->content_type->type, MU_MIME_CONTENT_TYPE_MULTIPART) == 0);
 }
