@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2020 Free Software Foundation, Inc.
+   Copyright (C) 2010-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -187,7 +187,7 @@ _mh_prop_fill (struct _mu_property *prop)
   if (rc)
     {
       if ((rc = mu_header_create (&header, NULL, 0)) != 0)
-	mu_error (_("cannot create context %s: %s"),
+	mu_error (_("cannot create property %s: %s"),
 		  mhprop->filename, mu_strerror (rc));
     }
   else
