@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2008-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2008-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -522,9 +522,9 @@ mu_m_server_begin (mu_m_server_t msrv)
     {
       /* Become a daemon. Take care to close inherited fds and to hold
 	 first three one, in, out, err   */
-      if (daemon (0, 0) < 0)
+      if ((rc = mu_daemon ()) != 0)
 	{
-	  mu_error (_("failed to become a daemon: %s"), mu_strerror (errno));
+	  mu_error (_("failed to become a daemon: %s"), mu_strerror (rc));
 	  exit (EXIT_FAILURE);
 	}
       mu_onexit_reset ();
@@ -919,7 +919,7 @@ static struct mu_cfg_param dot_server_cfg_param[] = {
     NULL, mu_offsetof (struct _mu_m_server,mode), _cb_daemon_mode,
     N_("Set operation mode."),
     /* TRANSLATORS: Words to the right of : are keywords. */
-    N_("mode: inetd|interactive|daemom") },
+    N_("mode: inetd|interactive|daemon") },
   { "foreground", mu_c_bool,
     NULL, mu_offsetof (struct _mu_m_server, foreground), NULL,
     N_("Run in foreground.") },
