@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2020 Free Software Foundation, Inc.
+   Copyright (C) 2011-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -174,7 +174,7 @@ __imap_msg_get_stream (struct _mu_imap_message *imsg, size_t msgno,
 	  mu_stream_set_buffer (imbx->cache, mu_buffer_full, 8192);
 	}
 
-      rc = mu_stream_size (imbx->cache, &imsg->offset);
+      rc = mu_stream_seek (imbx->cache, 0, MU_SEEK_END, &imsg->offset);
       if (rc)
 	return rc;
 
