@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2003, 2007, 2008, 2010 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,9 +12,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -31,15 +29,8 @@
 # include <strings.h>
 #endif
 
-#include <url0.h>
-#include <registrar0.h>
-
-static void url_pop_destroy (mu_url_t url);
-
-static void
-url_pop_destroy (mu_url_t url MU_ARG_UNUSED)
-{
-}
+#include <mailutils/sys/url.h>
+#include <mailutils/sys/registrar.h>
 
 /*
   POP URLs:
@@ -52,13 +43,6 @@ _url_pop_init (mu_url_t url)
 {
   if (url->port == 0)
     url->port = MU_POP_PORT;
-  
-  url->_destroy = url_pop_destroy;
-
-  /* not valid in pop url */
-  if (url->path || url->qargc || !url->host)
-    return EINVAL;
-
   return 0;
 }
 
@@ -73,13 +57,6 @@ _url_pops_init (mu_url_t url)
 {
   if (url->port == 0)
     url->port = MU_POPS_PORT;
-
-  url->_destroy = url_pop_destroy;
-
-  /* not valid in pops url */
-  if (url->path || url->qargc || !url->host)
-    return EINVAL;
-
   return 0;
 }
 
