@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002-2003, 2005-2012, 2014-2017 Free Software
-   Foundation, Inc.
+   Copyright (C) 2002-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -158,6 +157,7 @@ static struct mu_option options[] = {
   { "whatnowproc", 0, N_("PROG"), MU_OPTION_DEFAULT,
     N_("set the replacement for whatnow program"),
     mu_c_string, NULL, set_whatnowproc },
+  /* TRANSLATORS: "whatnowproc" is the variable name */
   { "nowhatnowproc", 0, NULL, MU_OPTION_DEFAULT,
     N_("don't run whatnowproc"),
     mu_c_int, &wh_env.nowhatnowproc, NULL, "1" },
@@ -178,7 +178,7 @@ static char default_format_str[] =
     "%(void Re:)%>) %(void(unre{subject}))%(trim)%(putstr)%>\n"
     "%(lit)%<(in_reply_to)%(void(width))%(printhdr In-reply-to: )\\n%>"
     "%(lit)%<(references)%(void(width))%(printhdr References: )\\n%>"
-    "X-Mailer: MH (%(package_string))\n"
+    "User-Agent: MH (%(package_string))\n"
     "--------\n";
 
 void
