@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2012 Free Software Foundation, Inc.
+   Copyright (C) 2009-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -29,7 +29,7 @@ api_strerror (PyObject *self, PyObject *args)
     return NULL;
 
   str = (char *)mu_strerror (status);
-  return _ro (PyString_FromString (str));
+  return _ro (PyUnicode_FromString (str));
 }
 
 static PyMethodDef methods[] = {
@@ -40,8 +40,16 @@ static PyMethodDef methods[] = {
   { NULL, NULL, 0, NULL }
 };
 
+static struct PyModuleDef moduledef = {
+  PyModuleDef_HEAD_INIT,
+  PY_MODULE,
+  NULL,
+  -1,
+  methods
+};
+
 void
 _mu_py_attach_error (void)
 {
-  _mu_py_attach_module (PY_MODULE, methods);
+  _mu_py_attach_module (&moduledef);
 }
