@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2012, 2014-2016 Free Software Foundation, Inc.
+   Copyright (C) 2009-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library.  If not, see 
+   Public License along with this library.  If not, see
    <http://www.gnu.org/licenses/>. */
 
 #include "libmu_py.h"
@@ -26,49 +26,18 @@ _repr1 (PyObject *self)
 {
   char buf[80];
   sprintf (buf, "<" PY_MODULE "." PY_CSNAME1 " instance at %p>", self);
-  return PyString_FromString (buf);
+  return PyUnicode_FromString (buf);
 }
 
 static PyTypeObject PyMailcapType = {
-  PyObject_HEAD_INIT(NULL)
-  0,                         /* ob_size */
-  PY_MODULE "." PY_CSNAME1,  /* tp_name */
-  sizeof (PyMailcap),        /* tp_basicsize */
-  0,                         /* tp_itemsize */
-  (destructor)_py_dealloc,   /* tp_dealloc */
-  0,                         /* tp_print */
-  0,                         /* tp_getattr; __getattr__ */
-  0,                         /* tp_setattr; __setattr__ */
-  0,                         /* tp_compare; __cmp__ */
-  _repr1,                    /* tp_repr; __repr__ */
-  0,                         /* tp_as_number */
-  0,                         /* tp_as_sequence */
-  0,                         /* tp_as_mapping */
-  0,                         /* tp_hash; __hash__ */
-  0,                         /* tp_call; __call__ */
-  _repr1,                    /* tp_str; __str__ */
-  0,                         /* tp_getattro */
-  0,                         /* tp_setattro */
-  0,                         /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT,        /* tp_flags */
-  "",                        /* tp_doc */
-  0,                         /* tp_traverse */
-  0,                         /* tp_clear */
-  0,                         /* tp_richcompare */
-  0,                         /* tp_weaklistoffset */
-  0,                         /* tp_iter */
-  0,                         /* tp_iternext */
-  0,                         /* tp_methods */
-  0,                         /* tp_members */
-  0,                         /* tp_getset */
-  0,                         /* tp_base */
-  0,                         /* tp_dict */
-  0,                         /* tp_descr_get */
-  0,                         /* tp_descr_set */
-  0,                         /* tp_dictoffset */
-  0,                         /* tp_init */
-  0,                         /* tp_alloc */
-  0,                         /* tp_new */
+  .ob_base = { PyObject_HEAD_INIT(NULL) },
+  .tp_name = PY_MODULE "." PY_CSNAME1,
+  .tp_basicsize = sizeof (PyMailcap),
+  .tp_dealloc = (destructor)_py_dealloc,
+  .tp_repr = _repr1,
+  .tp_str = _repr1,
+  .tp_flags = Py_TPFLAGS_DEFAULT,
+  .tp_doc = "",
 };
 
 PyMailcap *
@@ -82,49 +51,18 @@ _repr2 (PyObject *self)
 {
   char buf[80];
   sprintf (buf, "<" PY_MODULE "." PY_CSNAME2 " instance at %p>", self);
-  return PyString_FromString (buf);
+  return PyUnicode_FromString (buf);
 }
 
 static PyTypeObject PyMailcapEntryType = {
-  PyObject_HEAD_INIT(NULL)
-  0,                         /* ob_size */
-  PY_MODULE "." PY_CSNAME2,  /* tp_name */
-  sizeof (PyMailcapEntry),   /* tp_basicsize */
-  0,                         /* tp_itemsize */
-  (destructor)_py_dealloc,   /* tp_dealloc */
-  0,                         /* tp_print */
-  0,                         /* tp_getattr; __getattr__ */
-  0,                         /* tp_setattr; __setattr__ */
-  0,                         /* tp_compare; __cmp__ */
-  _repr2,                    /* tp_repr; __repr__ */
-  0,                         /* tp_as_number */
-  0,                         /* tp_as_sequence */
-  0,                         /* tp_as_mapping */
-  0,                         /* tp_hash; __hash__ */
-  0,                         /* tp_call; __call__ */
-  _repr2,                    /* tp_str; __str__ */
-  0,                         /* tp_getattro */
-  0,                         /* tp_setattro */
-  0,                         /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT,        /* tp_flags */
-  "",                        /* tp_doc */
-  0,                         /* tp_traverse */
-  0,                         /* tp_clear */
-  0,                         /* tp_richcompare */
-  0,                         /* tp_weaklistoffset */
-  0,                         /* tp_iter */
-  0,                         /* tp_iternext */
-  0,                         /* tp_methods */
-  0,                         /* tp_members */
-  0,                         /* tp_getset */
-  0,                         /* tp_base */
-  0,                         /* tp_dict */
-  0,                         /* tp_descr_get */
-  0,                         /* tp_descr_set */
-  0,                         /* tp_dictoffset */
-  0,                         /* tp_init */
-  0,                         /* tp_alloc */
-  0,                         /* tp_new */
+  .ob_base = { PyObject_HEAD_INIT(NULL) },
+  .tp_name = PY_MODULE "." PY_CSNAME2,
+  .tp_basicsize = sizeof (PyMailcapEntry),
+  .tp_dealloc = (destructor)_py_dealloc,
+  .tp_repr = _repr2,
+  .tp_str = _repr2,
+  .tp_flags = Py_TPFLAGS_DEFAULT,
+  .tp_doc = "",
 };
 
 PyMailcapEntry *
@@ -147,12 +85,17 @@ api_mailcap_create (PyObject *self, PyObject *args)
 
   if (!PyStream_Check ((PyObject *)py_stm))
     {
-      PyErr_SetString (PyExc_TypeError, py_stm->ob_type->tp_name);
+      PyErr_SetString (PyExc_TypeError, ((PyObject*)py_stm)->ob_type->tp_name);
       return NULL;
     }
 
-  status = mu_mailcap_create (&py_mc->mc, py_stm->stm);
-  return _ro (PyInt_FromLong (status));
+  status = mu_mailcap_create (&py_mc->mc);
+  if (status)
+    return _ro (PyLong_FromLong (status));
+  status = mu_mailcap_parse (py_mc->mc, py_stm->stm, NULL);
+  if (status == MU_ERR_PARSE)
+    status = 0; /* FIXME */
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -177,8 +120,8 @@ api_mailcap_entries_count (PyObject *self, PyObject *args)
   if (!PyArg_ParseTuple (args, "O!", &PyMailcapType, &py_mc))
     return NULL;
 
-  status = mu_mailcap_entries_count (py_mc->mc, &count);
-  return status_object (status, PyInt_FromSize_t (count));
+  status = mu_mailcap_get_count (py_mc->mc, &count);
+  return status_object (status, PyLong_FromSize_t (count));
 }
 
 static PyObject *
@@ -188,7 +131,6 @@ api_mailcap_get_entry (PyObject *self, PyObject *args)
   Py_ssize_t i;
   PyMailcap *py_mc;
   PyMailcapEntry *py_entry = PyMailcapEntry_NEW ();
-
   if (!PyArg_ParseTuple (args, "O!n", &PyMailcapType, &py_mc, &i))
     return NULL;
   ASSERT_INDEX_RANGE (i, "mailcap");
@@ -198,6 +140,22 @@ api_mailcap_get_entry (PyObject *self, PyObject *args)
   return status_object (status, (PyObject *)py_entry);
 }
 
+static PyObject *
+api_mailcap_find_entry (PyObject *self, PyObject *args)
+{
+  int status;
+  PyMailcap *py_mc;
+  PyMailcapEntry *py_entry = PyMailcapEntry_NEW ();
+  char *name;
+
+  if (!PyArg_ParseTuple (args, "O!s", &PyMailcapType, &py_mc, &name))
+    return NULL;
+  status = mu_mailcap_find_entry (py_mc->mc, name, &py_entry->entry);
+
+  Py_INCREF (py_entry);
+  return status_object (status, (PyObject *)py_entry);
+}
+
 static PyObject *
 api_mailcap_entry_fields_count (PyObject *self, PyObject *args)
 {
@@ -209,53 +167,53 @@ api_mailcap_entry_fields_count (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_mailcap_entry_fields_count (py_entry->entry, &count);
-  return status_object (status, PyInt_FromSize_t (count));
+  return status_object (status, PyLong_FromSize_t (count));
 }
 
 static PyObject *
 api_mailcap_entry_get_field (PyObject *self, PyObject *args)
 {
   int status;
-  Py_ssize_t i;
-  char buf[256];
+  char *name;
+  char const *value;
   PyMailcapEntry *py_entry;
 
-  if (!PyArg_ParseTuple (args, "O!n", &PyMailcapEntryType, &py_entry, &i))
+  if (!PyArg_ParseTuple (args, "O!s", &PyMailcapEntryType, &py_entry, &name))
     return NULL;
-  ASSERT_INDEX_RANGE (i, "mailcap");
-  status = mu_mailcap_entry_get_field (py_entry->entry, i, buf,
-				       sizeof (buf), NULL);
-  return status_object (status, PyString_FromString (buf));
+  status = mu_mailcap_entry_sget_field (py_entry->entry, name, &value);
+  return status_object (status,
+			status == 0
+			  ? (value ? PyUnicode_FromString (value)
+				   : PyBool_FromLong (1))
+			  : PyBool_FromLong (0));
 }
 
 static PyObject *
 api_mailcap_entry_get_typefield (PyObject *self, PyObject *args)
 {
   int status;
-  char buf[256];
+  char const *value;
   PyMailcapEntry *py_entry;
 
   if (!PyArg_ParseTuple (args, "O!", &PyMailcapEntryType, &py_entry))
     return NULL;
 
-  status = mu_mailcap_entry_get_typefield (py_entry->entry, buf,
-					   sizeof (buf), NULL);
-  return status_object (status, PyString_FromString (buf));
+  status = mu_mailcap_entry_sget_type (py_entry->entry, &value);
+  return status_object (status, PyUnicode_FromString (status == 0 ? value : ""));
 }
 
 static PyObject *
 api_mailcap_entry_get_viewcommand (PyObject *self, PyObject *args)
 {
   int status;
-  char buf[256];
+  char const *value;
   PyMailcapEntry *py_entry;
 
   if (!PyArg_ParseTuple (args, "O!", &PyMailcapEntryType, &py_entry))
     return NULL;
 
-  status = mu_mailcap_entry_get_viewcommand (py_entry->entry, buf,
-					     sizeof (buf), NULL);
-  return status_object (status, PyString_FromString (buf));
+  status = mu_mailcap_entry_sget_command (py_entry->entry, &value);
+  return status_object (status, PyUnicode_FromString (status == 0 ? value : ""));
 }
 
 static PyMethodDef methods[] = {
@@ -271,6 +229,9 @@ static PyMethodDef methods[] = {
   { "get_entry", (PyCFunction) api_mailcap_get_entry, METH_VARARGS,
     "Return in 'entry' the mailcap entry of 'no'." },
 
+  { "find_entry", (PyCFunction) api_mailcap_find_entry, METH_VARARGS,
+    "Return in 'entry' the mailcap entry for given content-type." },
+
   { "entry_fields_count", (PyCFunction) api_mailcap_entry_fields_count,
     METH_VARARGS,
     "Return the number of fields found in the entry." },
@@ -290,6 +251,14 @@ static PyMethodDef methods[] = {
   { NULL, NULL, 0, NULL }
 };
 
+static struct PyModuleDef moduledef = {
+  PyModuleDef_HEAD_INIT,
+  PY_MODULE,
+  NULL,
+  -1,
+  methods
+};
+
 int
 mu_py_init_mailcap (void)
 {
@@ -306,7 +275,7 @@ void
 _mu_py_attach_mailcap (void)
 {
   PyObject *m;
-  if ((m = _mu_py_attach_module (PY_MODULE, methods)))
+  if ((m = _mu_py_attach_module (&moduledef)))
     {
       Py_INCREF (&PyMailcapType);
       Py_INCREF (&PyMailcapEntryType);
